'use strict';

const _ = require('lodash');

class PubsubSubscription {

    constructor(id, topic, options) {
        this.id = id;
        this.topic = topic;
        this.options = _.defaults(options || {}, {
            maxInProgress: 1,
            ackDeadlineSeconds: 60,
            expire: false,
            autoAck: false,
            interval: undefined,
        });

        this.listenCallback = null;
        this.subscription = null;
    }

    getSubscription() {
        if(!this.subscription) {
            this.subscription = this.create(this.id);
        }

        return this.subscription;
    }

    create(id) {
        return this.topic
            .getTopic()
            .then(topic => {
                let subscription = topic.subscription(id, this.options);

                let opts = {
                    autoCreate: true,
                    maxInProgress: this.options.maxInProgress,
                    ackDeadlineSeconds: this.options.ackDeadlineSeconds,
                    autoAck: this.options.autoAck,
                    interval: this.options.interval,
                };

                return subscription
                    .get(opts)
                    .then(data => data[0]);
            });
    }

    listen(msgCallback) {
        if(this.listenCallback) {
            return Promise.reject(new Error('already listening for messages'));
        }

        return this
            .getSubscription()
            .then(subs => {
                let callback = (msg) => {
                    // Called every time a message is received.

                    // message.id = ID of the message.
                    // message.ackId = ID used to acknowledge the message receival.
                    // message.data = Contents of the message.
                    // message.attributes = Attributes of the message.
                    // message.timestamp = Timestamp when Pub/Sub received the message.

                    // Ack the message:
                    // message.ack(callback);

                    // Skip the message. This is useful with `maxInProgress` option when
                    // creating your subscription. This doesn't ack the message, but allows
                    // more messages to be retrieved if your limit was hit.
                    // message.skip();

                    msgCallback.apply(this, [msg]);
                };

                subs.on('message', callback);
                this.listenCallback = callback;
            });
    }

    stop() {
        if(!this.listenCallback) {
            return Promise.reject(new Error('is not listening for messages'));
        }

        return this
            .getSubscription()
            .then(subs => {

                subs.removeListener('message', this.listenCallback);
                this.listenCallback = null;
            });
    }

    isListening() {
        return !!this.listenCallback;
    }

    destroy() {
        if(!this.subscription) {
            return Promise.resolve();
        }

        let promise = null;

        if(this.isListening()) {
            promise = this.stop();

        } else {
            promise = Promise.resolve();
        }

        return promise
            .then(() => {
                return this
                    .subscription.then(subs => {
                        return subs
                            .delete()
                            .then(data => true);
                    });
            })
            .then(() => {
                this.subscription = null;
            });
    }
}

module.exports = PubsubSubscription;