'use strict';

const chai = require('chai');
const mocha = require('mocha');
const should = require('should');

const PubsubManager = require('../index');

describe('Pubsub', function() {
    this.timeout(10 * 1000);

    let pubsub = null;

    let topicId = 'test';
    let subscriptionId = 'test';

    /* -------------------------------------------------------------------------
     * Utils
     * ---------------------------------------------------------------------- */

    function defer() {
        let resolve, reject;
        let promise = new Promise(function() {
            resolve = arguments[0];
            reject = arguments[1];
        });
        return {
            resolve: resolve,
            reject: reject,
            promise: promise
        };
    }

    /* -------------------------------------------------------------------------
     * Before
     * ---------------------------------------------------------------------- */

    before(function() {
        let options = {
            credentials: {
                projectId: process.env.CRED_PROJECTID,
                keyFilename: process.env.CRED_KEYFILE,
            }
        };

        pubsub = new PubsubManager(options);
    });

    /* -------------------------------------------------------------------------
     * Create
     * ---------------------------------------------------------------------- */

    describe('Topic', () => {
        it('shoud create a new topic', () => {
            return pubsub
                .getTopic(topicId)
                .getTopic();
        });

        it('should reuse a previously created topic', () => {
            return pubsub
                .getTopic(topicId)
                .getTopic();
        });
    });

    /* -------------------------------------------------------------------------
     * Subscribe
     * ---------------------------------------------------------------------- */

    describe('Subscriptions', () => {
        it('should create a new subscription', () => {
            return pubsub
                .getTopic(topicId)
                .subscribe(subscriptionId)
                .getSubscription();
        });

        it('should publish a message', () => {
            return pubsub
                .getTopic(topicId)
                .publish({
                    hello: true
                })
                .then(ids => {
                    (ids).should.be.an.Array();
                });
        });

        it('should start listen and read a message', () => {
            let topic = pubsub.getTopic(topicId);

            let sub = topic.subscribe(subscriptionId);

            let deferred = defer();

            return sub
                .listen(msg => {
                    sub.stop();
                    deferred.resolve(msg);
                })
                .then(() => deferred.promise);
        });
    });

    /* -------------------------------------------------------------------------
     * Schedule and repeat
     * ---------------------------------------------------------------------- */

    describe.skip('Schedule and repeat', () => {
        it('should delay a subscription', () => {
            let topic = pubsub.getTopic(topicId);

            topic
                .publish({
                    published_at: +Date.now(),
                }, {
                    delay: 3,
                });

            let sub = topic.subscribe(subscriptionId);

            let deferred = defer();

            return sub
                .listen(msg => {
                    sub.stop();
                    deferred.resolve(msg);
                })
                .then(() => deferred.promise)
                .then(msg => {
                    let elapsed = (Date.now() - msg.data.published_at) / 1000;
                    (elapsed).should.be.above(3);
                });
        });
    });

    /* -------------------------------------------------------------------------
     * Clean
     * ---------------------------------------------------------------------- */

    describe('Clean', () => {
        it('should delete a subscription', () => {
            return pubsub
                .getTopic(topicId)
                .subscribe(subscriptionId)
                .destroy();
        });

        it('should delete a topic', () => {
            return pubsub
                .getTopic(topicId)
                .destroy();
        });
    });
});